/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/*****************************************************************************
  program name: lm_auto.c

     Version to try to adapt to a quantitative trait.

******************************************************************************/

/**
 * @defgroup lm_auto lm_auto
 * @ingroup autozyg
 * @{
 */

#define PNAM  "lm_auto"

#include "autozyg.h"
#include "auto_chk.c"

#define AUTO_CKPNTS 1000

void  do_auto_sweep (MSD_PTR, int);
void  do_scores (int);
void  score_gametes (int);
void  output_scores (int);
void  print_probs (int, int);
void  print_windows (int, int);
void  cmp_score_xact (int);
double ***ragged_alloc_dbl3d (int, int, int **);

static FILE *oScorF = NULL;

static int chkitvl = AUTO_CKPNTS;

static int scored_iterations;           /* Number of scored iterations.       */

static int mc_itno;

static int *gaga;                       /* [g]                            */
static int *npt;                        /* [g+1]  no. of patterns for
                                         *        gametes                 */
static int **iindex;                    /* [g+1][cd+1]                    */
static int **label;                     /* [g+1][pt+1]                    */
static int **list;                      /* [g+1][pt+1]                    */
static int nwpt;                        /* number of binary patterns,
                                         *   and of windows               */
static int *gw, *wloc;                  /* binary return for locus in
                                         *   window                       */
static double ****wscore;               /* window scores, each cmpt,
                                         * each scoreset, each window,
                                         *   each pattern
                                         *   assume 1 compnt, 1 scoreset  */
static double ***xscore = NULL;         /* [ns][nl][pt] exact prob scores */
static int **count;                     /* [nc][nl]                       */
static double ****nscore;               /* [nc][ns][nl][pt]  ibd counts:
                                         *   ea component, ea scoreset,
                                         *   ea locus, ea pattern         */

static double *sss;                     /* [nl]                           */
static double ss;

static double *ctot;                    /* CDLL by component.                 */
static double **prob;

int main (int argc, char *argv[])
{
   FILE *ibdF = NULL;
   FILE *oSeedF;

   MSD_PTR p_msd;

   int   output_iterations;             /* Total number of iterations in      */
   /*                                      which lm_auto is to add output to  */
   /*                                      the scores file.                   */

   int   nsw;
   int   nCompnt1;                      /* Number of components to be sampled */
   /*                                      from the exact distribution.       */
   int   nctemp;                        /* Backup copy of original n_compnts. */
   int   comp_no;                       /* Current component number.          */
   int   iteration;
   int   find_exact_samp_probs;
   int   i1;

   /* ................................  SET-UP   ............................ */

   nsw = setupAutozyg (argc, argv, PNAM, 0);

   /* The following check is being added to lm_auto because gl_auto runs as   */
   /* lm_auto and gl_auto supports the "set mcmc markers only" statement,     */
   /* which is not implemented for lm_auto.                                   */

   if (McmcMarkersOnly == TRUE)
   {
      fprintf (stderr, " =================================  Warning");
      fprintf (stderr, "  ==================================\n");
      fprintf (stderr, " The \"set mcmc markers only\" statement is not");
      fprintf (stderr, " implemented for program \"lm_auto\"\n");
      fprintf (stderr, " ==========================================");
      fprintf (stderr, "====================================\n\n\n");
      fflush (stderr);
   }

   setupSampling ();

   nCompnt1 = 0;
   if (!GlobalMcmc)
   {
      /* Count the number of components for which sampling from the exact     */
      /* distribution is to be done.                                          */
      for (comp_no = 1; comp_no <= nCompnt; ++comp_no)
         if (cmplist[comp_no].nmeioses <= MaxNMexact)
            ++nCompnt1;
   }

#if 0
   /* The call to setupInherit was moved down, so we can know if sampling     */
   /* from the exact distribution is to be done for any of the components.    */

   if (nCompnt1 == 0)
      prob = setupInherit (McmcMarkersOnly);
   else
#endif
      prob = setupInherit (0);

   if (MMSampler == Y)
   {
      /* Set up for the multiple-meiosis sampler.                             */
      Max_update = 8;
      P_meio = 0.2;
      P_ind = 0.3;
      P_sib = 0.3;
      P_3gen = 0.2;

      setup_marriage ();
      prnt_marriage ();
   }

   else if (MMSampler == N)
   {
      /* Set up for the old M-sampler.                                        */
      Max_update = 1;
      P_meio = 1.0;
      P_ind = 0.0;
      P_sib = 0.0;
      P_3gen = 0.0;
   }

   /* Set up probands for gene scoring.                                       */
   /* If the number of components read is less than the last one for which    */
   /* there are proband gametes, reset the number of components to score.     */

   NScCmp = set_probgam (n_compnts);

   /* gaga[], iindex[][], label[][], list[][], npt[] are malloced for the     */
   /* number of gametes in the largest set.                                   */
   /* Allocation of count and sss moved here by EAT in February of 2002.      */

   alloc_ibd (&gaga, &iindex, &label, &list, &npt, &count, &nscore, &sss);

   /* Count total number of patterns.                                         */
   /* Modified by EAT in March of 2001.  Now will do only full set of         */
   /* patterns for each number of gametes.                                    */

   for (i1 = 1; i1 <= MxGam; ++i1)
      npt[i1] = index_ibd (i1, 0, 0, iindex[i1], label[i1], list[i1], ibdF);

   pr_npatsets (npt);

   nwpt = 1;
   for (i1 = 0; i1 < NLocWndw; ++i1)
      nwpt *= 2;

   if (NLocWndw)
      alloc_wndw (&gw, &wloc, &wscore, NWndw, nwpt);
   else
   {
      wloc = malloc ((NLoci + 1) * sizeof (int));
      alloc_chk (WHERE, wloc);
   }

   /* Open output scores file, if requested to do so.                         */

   if (FilIt && !oScorF)
      if (!(oScorF = open_fil (OSCORE)))
         give_up (ERR_qit);

   /* Calculate the total number of iterations in which lm_auto is to add     */
   /* output to the scores file.                                              */

   output_iterations = NIt / (ScorIt * max (FilIt, 1));

   if (PrintLevel >= 4)
   {
      ctot = malloc (n_compnts * sizeof (double));
      alloc_chk (WHERE, ctot);

      p_msd = alloc_msd ();
      alloc_logprob ();
   }

   if (!GlobalMcmc)
   {
      /* Backup actual number of components in nctemp, so that n_compnts can  */
      /* be set to 1.  We want lm_auto to behave as if each current component */
      /* is the only component in the pedigree.                               */

      nctemp = n_compnts;
      n_compnts = 1;

      for (comp_no = 1; comp_no <= nctemp; ++comp_no)
      {
         if (PrintLevel >= 4 && comp_no > 1)
         {
            /* Initialize variables which need to be zeroed for each new      */
            /* component being sampled.  Note that p_msd->temp_log_prob_seg   */
            /* does not need to be zeroed.                                    */

            zero_mcmc_stats (p_msd);
         }

         scored_iterations = 0;         /*Initialize scored iterations count. */

         printf ("\n ====== IBD scores for component %d are estimated using",
                 comp_no);

         if (cmplist[comp_no].nmeioses <= MaxNMexact)
         {
            find_exact_samp_probs = TRUE;

            printf (" IID exact sampling ======\n\n\n");
            fflush (stdout);

            /* Forward computation.                                           */
            exact_smpl (1, 0, comp_no, 0, NMarks, nFGL, prob, NMarks, ctot);

            /* Perform IID sampling backwards.                                */
            for (iteration = 1; iteration <= output_iterations; ++iteration)
            {
               exact_smpl (0, 1, comp_no, 0, NMarks, nFGL, prob, NMarks, ctot);

               do_scores (comp_no);

               if (PrintLevel >= 4)
                  update_counts ("C", p_msd, comp_no, 0, prob, ctot);

               ++scored_iterations;

               /* Tests to see whether or not to output to scores file.       */
               if (FilIt && ((scored_iterations % FilIt) == 0))
                  output_scores (comp_no);
            }

            /* Compute IBD probabilities using the IID sample from the exact  */
            /* distribution for the current component.                        */

            cmp_score_xact (comp_no);
         }

         else
         {
            /* cmplist[comp_no].nmeioses > MaxNMexact, so sampling is to be   */
            /* done separately for this component.                            */

            setup_sampling_cmp (comp_no, MMSampler, NMarks);

            find_exact_samp_probs = FALSE;

            printf (" MCMC ======\n\n\n");

            prntStatusBMC ();           /* Print status for beginning MCMC.   */
            fflush (stdout);

            /* ................  BURN-IN  .................................   */

            /* Now do burn-in iterations.                                     */

            printf (" Now starting %d iterations of burn-in\n", NBrnIt);
            for (iteration = 1; iteration <= NBrnIt; ++iteration)
               sampling_cmp (iteration, comp_no, 0, MMSampler, NMarks, nFGL,
                             prob);
            printf ("\n");
            fflush (stdout);

            /* Now do the regular MCMC sampling, and score the results.       */

            printf (" Now starting %d iterations of MCMC\n", NIt);
            fflush (stdout);

            for (mc_itno = 1; mc_itno <= NIt; ++mc_itno)
               do_auto_sweep (p_msd, comp_no);

            /* Print status that we have finished with the MCMC.              */
            printf ("\n\n");
            prntStatusFMC ();
            fflush (stdout);

            free_sampling_cmp ();
         }

         /* Now print out the sampling statistics for the current component.  */

         pr_cmpnthead (comp_no - 1);
         pr_comp_mcmc_stats (p_msd, 0, scored_iterations, comp_no, 0);
         fflush (stdout);

         /* Now print the results for the current component.                  */

         print_probs (comp_no, find_exact_samp_probs);

         /* If xscore has been allocated in order to find IBD probabilities   */
         /* using a sample from the exact distribution for the current        */
         /* component, it is no longer needed for this component, so free it. */

         if (xscore)
            free_3d ((void *) &xscore);
      }

      /* We have now completed IBD probability computations for each separate */
      /* component, so restore true number of components to n_compnts.        */

      n_compnts = nctemp;
   }

   else if (GlobalMcmc)
   {
      /* Setup variables for LM-multiple sampling: LPT Apr 2006.              */
      setup_sampling_cmp (0, MMSampler, NMarks);

      find_exact_samp_probs = FALSE;

      /* Try to restart if a rescue file is specified.                        */

      iteration = 0;
      auto_restart (nscore, p_msd->tot_log_prob_pen, p_msd->tot_log_prob_seg,
                    prob, count, p_msd->recounts, &scored_iterations,
                    &iteration);

      /* ................  BURN-IN  .......................................   */

      /* If this is not a restart run of lm_auto, do the burn-in iterations.  */

      if (iteration == 0)
      {
         prntStatusBMC ();              /* Print status for beginning MCMC.   */
         fflush (stdout);

         printf (" Now starting %d iterations of burn-in\n", NBrnIt);
         for (iteration = 1; iteration <= NBrnIt; ++iteration)
            sampling_cmp (iteration, 0, 0, MMSampler, NMarks, nFGL, prob);
         printf ("\n");
         fflush (stdout);

         scored_iterations = 0;         /*Initialize scored iterations count. */
      }

      /* ................  COMPUTE USING MCMC  ............................   */

      /* Now do the regular MCMC sampling, and score the results.             */

      printf (" Now starting %d iterations of MCMC\n", NIt);
      fflush (stdout);

#ifdef AUTO_DISP                        /* May not want to link graphics fncs */
      if (GDispOpt)
      {
         i1 = 0;
         while (!NGDispNams[i1])
            ++i1;
         getIndIxs (i1 + 1, NGDispNams[i1], GDispNam[i1], GDispIx[i1]);
         glutInit (&argc, argv);        /* initialize "glut" for X Windows    */
         gfmInitGFM ();                 /* set up display callbacks           */
         glutMainLoop ();               /* start graphics; idle callback      */
      }                                 /*   function calls do_auto_sweep ()  */
#else
      if (GDispOpt)
         give_up (ERA_ngd);
#endif

      if (!GDispOpt)                    /* Ignore graphics.                   */
         for (mc_itno = 1; mc_itno <= NIt; ++mc_itno)
            do_auto_sweep (p_msd, 0);

      /* Print status that we have finished with the MCMC.                    */
      printf ("\n\n");
      prntStatusFMC ();
      fflush (stdout);

      free_sampling_cmp ();

      /* Print out the sampling statistics and results for each component.    */

      for (comp_no = 1; comp_no <= n_compnts; ++comp_no)
      {
         pr_cmpnthead (comp_no - 1);
         pr_comp_mcmc_stats (p_msd, 0, scored_iterations, comp_no, comp_no - 1);
         fflush (stdout);

         print_probs (comp_no, find_exact_samp_probs);
      }
   }

   /* Free up allocated memory.                                               */
   free_rescue_filenames (orescName);
   free_xxgg ();

   free_ibd (&gaga, &iindex, &label, &list, &npt, &count, &nscore, &sss);
   if (NLocWndw)
      free_wndw (&gw, &wloc, &wscore, NWndw, nwpt);
   else
      free (wloc);

   if (PrintLevel >= 4)
   {
      free_1d ((void *) &ctot);
      free_msd (p_msd);
      free_logprob ();
   }

   locus_peel_free ();
   free_allocate_loci ();
   free_chromo (0);
   free_gen_pen ();
   free_seg_pen (NMarks + 1, &prob);
   free_msmpl ();
   free_traits ();

   if (SamplSeedsOut)
   {
      getSeeds (SAMPL);
      if (!(oSeedF = open_fil (OSEED)))
         give_up (ERR_qit);
      writSeeds (oSeedF);
      fclose (oSeedF);
   }
   if (FilIt)
      fclose (oScorF);

   if (Scan)
      free_perm ();

   exit (EXIT_SUCCESS);
}

void do_auto_sweep (MSD_PTR p_msd, int cmp)
{
   int   comp_no;                       /* Current component number.          */

   /* Now do the next MCMC sampling iteration.                                */

   sampling_cmp (mc_itno, cmp, 0, MMSampler, NMarks, nFGL, prob);

   if (DebugMain)
   {
      fprintf (stderr, "\nAt iteration %d,\n", mc_itno);
      dbgXXGG ();
   }

   if ((mc_itno % ScorIt) == 0)
   {
      /* If the iteration number is a multiple of the scoring interval, then  */
      /* calculate the current interation's scores.                           */

      do_scores (cmp);

      if (PrintLevel >= 4)
         update_counts ("C", p_msd, cmp, 0, prob, ctot);

      ++scored_iterations;

      /* Tests to see whether or not to output to scores file.                */
      if (FilIt && ((scored_iterations % FilIt) == 0))
      {
         if (cmp > 0)
            output_scores (cmp);
         else if (cmp == 0)
         {
            for (comp_no = 1; comp_no <= n_compnts; ++comp_no)
               output_scores (comp_no);
         }
      }
   }

   if (chkitvl && ((mc_itno % chkitvl) == 0))
   {
      auto_dump (nscore, p_msd->tot_log_prob_pen, p_msd->tot_log_prob_seg, prob,
                 count, p_msd->recounts, scored_iterations, mc_itno);
   }

   return;
}

/******************************************************************************/
/*                                                                            */
/* Subroutine to add one to each count of the number of scores considered for */
/* IBD probability, before calling score_gametes to determine whether each    */
/* pair of gametes is IBD or not.  Added by SML in March of 2011.             */
/*                                                                            */
/******************************************************************************/

void do_scores (int cmp)
{
   int   c1;                            /* Current component subscript.       */
   int   p1;                            /* Current locus subscript.           */

   if (cmp > 0)
   {
      for (p1 = NMarks; p1 >= 0; --p1)
         ++count[cmp - 1][p1];

      score_gametes (cmp - 1);
   }

   else if (cmp == 0)
   {
      for (c1 = 0; c1 < n_compnts; ++c1)
         for (p1 = NMarks; p1 >= 0; --p1)
            ++count[c1][p1];

      for (c1 = 0; c1 < NScCmp; ++c1)   /* For each component and score set.  */
         score_gametes (c1);
   }

   return;
}

/******************************************************************************/
/*                                                                            */
/* Subroutine to score IBD probabilities for each pair of gametes in a given  */
/* component.  Added by SML in March of 2011.                                 */
/*                                                                            */
/******************************************************************************/

void score_gametes (int comp_ndx)
{
   int   s1;                            /* Current score set subscript.       */
   int   p1;                            /* Current locus subscript.           */

   /* For each score set in the selected component.                           */
   for (s1 = 0; s1 < NScSet[comp_ndx]; ++s1)
   {
      for (p1 = NMarks; p1 >= 0; --p1)  /* Score each locus.                   */
      {
         wloc[p1] = score_genes (chromo[0][p1], NPrGam[comp_ndx][s1], gaga,
                                 iindex[NPrGam[comp_ndx][s1]],
                                 list[NPrGam[comp_ndx][s1]],
                                 ProbIx[comp_ndx][s1],
                                 ProbGam[comp_ndx][s1],
                                 nscore[comp_ndx][s1][p1],
                                 NPtLab[comp_ndx][s1], PatLab[comp_ndx][s1]);
      }

      if (NLocWndw)
         score_windows (wscore[comp_ndx][s1], NLocWndw, wloc, NMarks);
   }

   return;
}

/******************************************************************************/
/*                                                                            */
/* The output_scores subroutine is called if a user has specified an "output  */
/* scores file".  This subroutine outputs for the component selected by the   */
/* first argument, comp_no, for each score set and proband gamete set the     */
/* interim IBD probabilities with their IBD label and pattern.  The code in   */
/* this subroutine was moved from do_auto_sweep so that it could be called    */
/* from more than one location.  This was done by SML in March of 2011.       */
/*                                                                            */
/******************************************************************************/

void output_scores (int comp_no)
{
   double IBD_prob;                     /* Computed IBD probability.          */

   int   comp_ndx;                      /* Current component subscript.       */
   int   s1;                            /* Current score set subscript.       */

   int   ng;                            /* Number of gamete sets.             */
   int   p2;                            /* Current pattern subscript.         */

   int   g1;                            /* Current gamete subscript.          */

   int   p1;                            /* Current locus subscript.           */
   int   p3;                            /* Locus subscript from revchro.      */

   comp_ndx = comp_no - 1;              /* Calculate component's subscript.   */

   /* Output the interim IBD probabilities for each score set defined for the */
   /* selected component.                                                     */

   for (s1 = 0; comp_ndx < NScCmp && s1 < NScSet[comp_ndx]; ++s1)
   {
      ng = NPrGam[comp_ndx][s1];        /* Get the number of proband gametes. */
      for (p2 = 0; p2 < npt[ng]; ++p2)  /* Output for each IBD pattern.       */
      {
         fprintf (oScorF, "%d ", comp_no); /* Output the component number.    */
         fprintf (oScorF, "%d ", ScSetNo[comp_ndx][s1]); /* Score set number. */

         invert_label (label[ng][p2], ng, gaga);
         fprintf (oScorF, "%d ", label[ng][p2]); /* Output the IBD label.     */
         fprintf (oScorF, " ");

         for (g1 = 0; g1 < ng; ++g1)
            fprintf (oScorF, "%d ", gaga[g1]); /* Output the IBD pattern.     */

         for (p1 = 0; p1 <= NMarks; ++p1)
         {
            /* Output the interim IBD probability at the current locus.       */
            p3 = revchro[0][p1];
            IBD_prob = nscore[comp_ndx][s1][p3][list[ng][p2]] /
               count[comp_ndx][p3];
            fprintf (oScorF, " %6.4f", IBD_prob);
         }

         fprintf (oScorF, "\n");
      }
   }
   fflush (oScorF);

   return;
}

/******************************************************************************/
/*                                                                            */
/* The print_probs subroutine may be called to print IBD probabilities for a  */
/* specific component.  The first argument, comp_no, is the component number  */
/* for which the IBD probabilities are to be printed.  The second argument,   */
/* print_exact_samp_probs, is either a zero or a one.  If it is a zero, then  */
/* IBD probabilities from MCMC are to be printed.  If it is a one, then IBD   */
/* probabilities from a sample from the exact distribution are to be printed. */
/* This subroutine was added by SML in March of 2011.                         */
/*                                                                            */
/******************************************************************************/

void print_probs (int comp_no, int print_exact_samp_probs)
{
   int   comp_ndx;                      /* Current component subscript.       */
   int   s1;                            /* Current score set subscript.       */
   int   ng;                            /* Number of gamete sets.             */
   int   p2;                            /* Current pattern subscript.         */
   int   p3;                            /* Current locus subscript.           */

   comp_ndx = comp_no - 1;              /* Calculate component's subscript.   */

   /* For each set, print probabilities for each pattern for each window.     */

   for (s1 = 0; comp_ndx < NScCmp && s1 < NScSet[comp_ndx]; ++s1)
   {
      /* Format and print headings.                                           */

      pr_probhead ();
      pr_pgamset (comp_ndx, s1);
      pr_lochead (chromo);

      /* Format and print data.                                               */

      ng = NPrGam[comp_ndx][s1];

      for (p2 = 0; p2 < npt[ng]; ++p2)
      {
         ss = 0.0;
         for (p3 = 0; p3 < NLoci; ++p3)
         {
            if (print_exact_samp_probs)
               sss[p3] = xscore[s1][p3][list[ng][p2]];
            else
               sss[p3] = nscore[comp_ndx][s1][p3][list[ng][p2]] /
                  count[comp_ndx][p3];
            ss += sss[p3];
         }

         if (ss > 0.0001)
         {
            invert_label (label[ng][p2], ng, gaga);
            pr_locprob (ng, gaga, sss, label[ng][p2]);
         }
      }
      printf ("\n");

      print_windows (comp_ndx, s1);     /* Check for output of window scores. */
      fflush (stdout);
   }

   return;
}

/******************************************************************************/
/*                                                                            */
/* If a user has asked for the scoring of IBD states for a moving window of   */
/* loci, the within window's IBD probability scores are printed by this       */
/* subroutine.  Added by SML in March of 2011.                                */
/*                                                                            */
/******************************************************************************/

void print_windows (int comp_ndx, int score_set)
{
   int   p2;                            /* Current pattern subscript.         */
   int   w1;                            /* Current window subscript.          */

   if (NLocWndw)                        /* Now output any window scores.      */
   {
      if (NPtLab[comp_ndx][score_set])
      {
         pr_prbwhead (comp_ndx, score_set);
         pr_wndwpat (comp_ndx, score_set);
         pr_wndwhead (NWndw);

         /* Format and print the data.                                        */

         for (p2 = 0; p2 < nwpt; ++p2)
         {
            /* Find the largest IBD probability score computed in any window. */
            ss = 0.0;
            for (w1 = 0; w1 < NWndw; ++w1)
            {
               if (wscore[comp_ndx][score_set][w1][p2] > ss)
                  ss = wscore[comp_ndx][score_set][w1][p2];
            }

            if (ss / NIt > 0.0001)
            {
               binary (p2, NLocWndw, gw);
               pr_wndwprob (comp_ndx, score_set, NWndw, p2, gw, wscore);
            }
         }

         printf ("\n");
      }
   }

   return;
}

/******************************************************************************/
/*                                                                            */
/* Subroutine for computing IBD probabilities using a sample from the exact   */
/* distribution for a specific component.  The argument, comp_no, is the      */
/* component number of the selected component.  This subroutine was added by  */
/* SML in March of 2011.                                                      */
/*                                                                            */
/******************************************************************************/

void cmp_score_xact (int comp_no)
{
   double **prob_fwd;                   /* Matrix of forward probabilities.   */

   int  *cum_bin_sums;                  /* Vector of cumulative binomial sums. */
   int   c1;                            /* Cumulative probabilities subscript. */

   int   comp_ndx;                      /* Selected component subscript.      */

   int **n_IBD_pts;                     /* Array of numbers of IBD patterns.  */
   int   s1;                            /* Current score set subscript.       */
   int   p1;                            /* Current locus subscript.           */

   int   nm;                            /* Number of meioses.                 */
   int   nm2;                           /* 2 ^ (number of meioses).           */

   int   m1;                            /* Within cmplist meiosis subscript.  */
   int   m2;                            /* Within meioses meiosis subscript.  */

   comp_ndx = comp_no - 1;              /* Calculate component's subscript.   */

   if (NScSet[comp_ndx] > 0)
   {
      /* Create a matrix of the numbers of IBD patterns for each score set,   */
      /* given the number of proband gametes for each score set.              */

      n_IBD_pts = alloc_int2d (NScSet[comp_ndx], NLoci);
      for (s1 = 0; s1 < NScSet[comp_ndx]; ++s1)
         for (p1 = 0; p1 < NLoci; ++p1)
            n_IBD_pts[s1][p1] = PTgg[NPrGam[comp_ndx][s1]] + 1;

      /* Allocate xscore[][][] and initialize the array to all zeroes.        */
      xscore = ragged_alloc_dbl3d (NScSet[comp_ndx], NLoci, n_IBD_pts);
      free_2d ((void *) &n_IBD_pts);

      nm = cmplist[comp_no].nmeioses;
      nm2 = pow2 (nm);

      prob_fwd = alloc_dbl2d (NLoci, nm2);
      exact_ivprob (comp_no, 0, NMarks, nFGL, prob_fwd);

      cum_bin_sums = alloc_int1d (nm);

      for (c1 = 0; c1 < nm2; ++c1)
      {
         /* Set segregation indicators for the component from cum_bin_sums.   */
         for (m1 = 0; m1 < nm; ++m1)
         {
            m2 = cmplist[comp_no].meioses[m1];
            if (meiosis[m2].sex)        /* Male meiosis.                      */
               xx1[0][(meiosis[m2].ind).index] = cum_bin_sums[m1];
            else                        /* Female meiosis.                    */
               xx0[0][(meiosis[m2].ind).index] = cum_bin_sums[m1];
         }

         /* Reset FGL's including locus 0.                                    */
         reset_FGL (0, comp_no);

         for (p1 = NMarks; p1 >= 0; --p1) /* Score each locus.                */
         {
            /* If the probability for this meiosis/locus is > 0.0, then:      */
            if (prob_fwd[p1][c1] > 0.0)
            {
               /* For each score set in the selected component.               */
               for (s1 = 0; s1 < NScSet[comp_ndx]; ++s1)
               {
                  score_xact (NPrGam[comp_ndx][s1], gaga,
                              iindex[NPrGam[comp_ndx][s1]],
                              list[NPrGam[comp_ndx][s1]],
                              ProbIx[comp_ndx][s1],
                              ProbGam[comp_ndx][s1],
                              xscore[s1][p1], prob_fwd[p1][c1]);
               }
            }
         }
         biadd (cum_bin_sums, nm, 1);
      }

      free_2d ((void *) &prob_fwd);
      free_1d ((void *) &cum_bin_sums);
   }

   return;
}

/******************************************************************************/
/*                                                                            */
/* The ragged_alloc_dbl3d subroutine is a slightly modified version of the    */
/* alloc_dbl3d subroutine located in the alloc_array.c file.  This version    */
/* uses the last argument, ragged_dim3, which needs to be a two-dimensional   */
/* integer array containing the respective numbers of double precision        */
/* floating point numbers which need to be allocated (and initialized) as the */
/* final ragged dimension of the resulting array.                             */
/*                                                                            */
/* In the future it might be a good idea to move this subroutine to the       */
/* alloc_array.c file, so that it would be available throughout Morgan.       */
/*                                                                            */
/******************************************************************************/

double ***ragged_alloc_dbl3d (int dim1, int dim2, int **ragged_dim3)
{
   int   dim3;
   int   x1, x2;
   double ***array3d, *next_addr;

   /* Allocate memory for a three dimensional array of double precision       */
   /* floating point numbers.  The first dimension will consist of pointers   */
   /* to the second dimension.                                                */
   array3d = (double ***) malloc (dim1 * sizeof (double **));
   alloc_chk (WHERE, array3d);

   /* The second dimension will consist of pointers to the third dimension.   */
   array3d[0] = (double **) malloc (dim1 * dim2 * sizeof (double *));
   alloc_chk (WHERE, array3d[0]);

   /* Fill in the pointers in the first dimension so that they point to the   */
   /* pointers which make up the second dimension.                            */
   for (x1 = 1; x1 < dim1; ++x1)
      array3d[x1] = array3d[x1 - 1] + dim2;

   /* Calculate the total number of double precision floating point numbers   */
   /* which need to be allocated.                                             */
   dim3 = 0;
   for (x1 = 0; x1 < dim1; ++x1)
      for (x2 = 0; x2 < dim2; ++x2)
         dim3 += ragged_dim3[x1][x2];

   /* The third dimension will consist of the actual double precision         */
   /* floating point number vectors.                                          */
   array3d[0][0] = (double *) malloc (dim1 * dim2 * dim3 * sizeof (double));
   alloc_chk (WHERE, array3d[0][0]);

   /* Fill in the pointers in the second dimension so that they point to the  */
   /* actual double precision floating point number vectors which make up the */
   /* third dimension.                                                        */
   next_addr = array3d[0][0];
   for (x1 = 0; x1 < dim1; ++x1)
      for (x2 = 0; x2 < dim2; ++x2)
      {
         array3d[x1][x2] = next_addr;
         next_addr += ragged_dim3[x1][x2];
      }

   /* Initialize all of the actual double precision floating point numbers to */
   /* zeroes.                                                                 */
   memset (array3d[0][0], 0.0, dim1 * dim2 * dim3 * sizeof (double));

   return array3d;
}
