#!/usr/bin/python
#
# pairwise.py
# 
# Chris Glazner
# chris.glazner@u.washington.edu
# Wed Apr 7 2010
#
# This script takes a file of "individuals" (pairs of haplotypes)
# and runs an ibd_haplo 4-haplotype analysis on each pair of individuals.
# Running pairwise analysis will eventually be incorporated 
# into the ibd_haplo program itself.
#
# To run the script, place it in the directory where you are doing the
# analysis and set the two variables below. Configuration
# and input files from a normal ibd_haplo run should work without
# modification. Run the script with:
# python pairwise.py [your original .par file]
#
# "python pairwise.py ibd_haplo.par" does an analysis on 
# all six pairs of individuals that can be formed
# from the four individuals in haplotypes.dat

# CHANGE THESE TO RUN SCRIPT
# of inviduals
indivs = 4
# path to ibd_haplo executable
ibdhaplopath = "../ibd_haplo"


###############################################

import sys
import os
from linecache import *

if len(sys.argv)!=2:
    print("Usage: python pairwise.py [.par file]")

parfile = open(sys.argv[1])
inputfiles = open(parfile.readline().split()[3].replace('"',''))

print("Creating temporary .par file")
temppar = open("temp.par","w")
temppar.write('input extra file "tempinput"\n')
temppar.write(parfile.readline())
temppar.close()

print("Creating temporary input file list")
tempinputfiles = open("tempinput","w")
datfile = open(inputfiles.readline().replace('\n',''),"r")
haplos = inputfiles.readline().replace('\n','')
tempinputfiles.write("temp.dat\n")
tempinputfiles.write("temphaps.txt\n")
tempinputfiles.write(inputfiles.readline())
inputfiles.close()
tempinputfiles.close()

print("Creating temporary .dat file")
tempdatfile = open("temp.dat","w")
tempdatfile.write(datfile.readline())
line = datfile.readline().split()
tempdatfile.write(" %d %s %s \n" % (1,line[1],line[2]))
#line = datfile.readline().split()
#tempdatfile.write(" %s %d %s %s \n" % (line[0],4,line[2],line[3]))
for line in datfile.readlines():
    tempdatfile.write(line)
tempdatfile.close()


for i in range(1,indivs+1):
    for j in range(i+1,indivs+1):
        print("Creating temporary haplotype data file for individuals %d and %d" % (i,j))
        temphaps = open("temphaps.txt","w")
        temphaps.write(getline(haplos,2*i-1))
        temphaps.write(getline(haplos,2*i))
        temphaps.write(getline(haplos,2*j-1))
        temphaps.write(getline(haplos,2*j))
        temphaps.close()

        os.system("%s temp.par" % ibdhaplopath)

print("Cleaning up temporary files")
os.system("rm tempinput temp.dat temp.par temphaps.txt")

