/*  Program to do a basic analysis of IBD inference with no LD */
/*  Up to 4 genomes, or genotypic pairs  */

/* Also includes options to simulate the ibd status of chromosomes    */
/*  Initially these will just be segregants  from first cousins */
/*  We will make the chromosomes 2 Morgans, so most have an IBD segment */

#define DEFINE_DATA
#define PNAM "phoenix"

#include "parsie.h"
#include "rans.h"
#include "ibd_haplo.h"

#include  "haplo_process.c"
#include "hap_fwd_bk_compute.c"

int main (int argc, char *argv[])
{

   int   i, j, k;
   int   ns, nph, nstmp;                /* number of states, and phenotypes */
   int   nhd;                           /* number of actual data haplotypes */
   /*                        (2 or 4 ) */
   int   nset;                          /* number of sets of data haplotypes */
   int   totset;                        /*  nset + number of extracted pairs */
   int   nhap;                          /* number of haplos, and start and  */
   /*   end of SNP block               */
   int   nsnps1, nsnps;                 /* number of SNPS in analysis       */
   double tot_length;                   /*  total length to be analyzed     */

   double *allefreq;                    /* allele frequencies at each marker */

   int **haplodat, *name;
   /* constructed haplos, and data haplotypes  */

   /* array of phasing data; first is for individuals, second is for pairs of indivs */
   int **phasdatindiv;
   int **phasdat;
   int   tmp1, tmp2;                    /* holders for swapping alleles */

   int   nloc, *hapdat;                 /* number of loci and haplo data  */
   double *snpdist, *snppos;            /* marker distances and positions  */

   double **qq_ibd, **qq_ibd_final;
   double **pprob;
   double ffibd[15];                    /* overall inbr,  marginal ibd prob */
   int  *phasptr = 0;

   char  compu_fname[60], hapdat_fname[60], mrks_fname[60], phas_fname[60];
   /* file names in XTRA */
   FILE *fptr, *fptr2;

   initMorgProg (argv);

   /* parse input parameter statements  */
   if (!proc_pars (argc, argv, PNAM))
      exit (EXIT_FAILURE);

   prntStatusPP ();

   printf
      (" === PHOENIX PROGRAM: NO CHECKING OF PARAMETER STATEMENTS === \n \n");

   if (!(fptr = open_fil (XTRA)))       /* we will now use this for filenames */
      give_up (ERR_qit);

   fscanf (fptr, "%s", compu_fname);

/* we will read nsnps SNPS taking best from blocks size nblk at gap ngap */
/*   at min freq qq */

   fptr2 = fopen (compu_fname, "r");
   fscanf (fptr2, "%d %*d %d %d", &cz, &igin, &igdx);
   if (igdx == 1)                       /*   genotypic analysis                 */
   {
      ns = 9;
      nph = 9;
   }
   else                                 /* 0 for haplotypic, 1 for partial phase */
   {
      ns = 15;
      nph = 16;
   }

   fscanf (fptr2, "%d %d %lf", &nset, &nhd, &tot_length);
   fscanf (fptr2, "%d %lf %lf %lf", &nsnps, &fkin, &ffrate, &delta);
   fclose (fptr2);

   printf ("\n Data check: from file %s", compu_fname);
   printf ("\n ns = %d, nph=%d, igin= %d, igdx=%d", ns, nph, igin, igdx);
   printf ("\n  nset= %d, nhd= %d, tot_length= %f", nset, nhd, tot_length);
   printf ("\n  nsnps=%d, fkin= %f, ffrate=%f, delta = %f\n",
           nsnps, fkin, ffrate, delta);
   fflush (stdout);

/* Now open and print the other file names  */

   fscanf (fptr, "%s", hapdat_fname);
   fscanf (fptr, "%s", mrks_fname);
   fscanf (fptr, "%s", phas_fname);
   fclose (fptr);

   printf ("Check names of input files  %s  ", compu_fname);
   printf ("%s  ", hapdat_fname);
   printf ("%s  ", mrks_fname);
   if (igdx == 2)
      printf ("%s  ", phas_fname);
   printf ("\n");

/************************* TEMP TEMP TEMP TEMP FIX ME *****/
   skipmark = 1;

   nloc = nsnps;                        /* number loci used in analysis  */
   nsnps1 = nsnps;                      /* number SNPs actually read in  */

   nhap = nhd * nset;
   if (igin)                            /* genotypes of individuals */
      name = malloc (nhap / 2 * sizeof (int));
   else                                 /* list of hap-"names"   */
      name = malloc (nhap * sizeof (int));

   snppos = malloc (nloc * sizeof (double));
   haplodat = alloc_int2d (nhap, nsnps);

   hapdat = malloc (nloc * sizeof (int)); /* All cases: analysis data */
   allefreq = malloc (nloc * sizeof (double));
   snpdist = malloc (nloc * sizeof (double));
   /* move penmat to be internal to hap_fwd and hap_bk subroutines */
   /* penmat = malloc (ns * sizeof (double)); */
   transmat = alloc_dbl2d (ns, ns);
   qq_ibd = alloc_dbl2d (nsnps, ns);
   pprob = alloc_dbl2d (nsnps, ns);

   printf ("\n now to read the real genetic data ");
   fflush (stdout);
   if (!(fptr = fopen (hapdat_fname, "r")))
      give_up (ERR_qit);

   if (igin)                            /* real data are input as genotypic */
   {
      for (i = 0; i < nhap; i = i + 2)
      {
         fscanf (fptr, "%d", &(name[i / 2]));
         for (j = 0; j < nsnps; ++j)
            fscanf (fptr, "%d %d", &(haplodat[i][j]), &(haplodat[i + 1][j]));
      }
   }
   else                                 /* data are input as haplotypic  */
   {
      for (i = 0; i < nhap; ++i)
      {
         fscanf (fptr, "%d", &(name[i]));
         for (j = 0; j < nsnps; ++j)
            fscanf (fptr, "%d", &(haplodat[i][j]));
      }
      for (i = 0; i < nhap; ++i)
      {
         printf ("\n %d", name[i]);
         for (j = 0; j < 10; ++j)
            printf (" %d", haplodat[i][j]);
      }
   }
   if (igdx == 2)
   {
      /* one line of phasing data per haplo pair */
      phasdatindiv = alloc_int2d (nhap / 2, nsnps);
      /* could ditch this matrix and just compute the vector for each compute run */
      phasdat = alloc_int2d (nset, nsnps);
      if (!(fptr = fopen (phas_fname, "r")))
         give_up (ERR_qit);
      printf ("\n reading phasing status data ");
      for (i = 0; i < nhap / 2; ++i)
      {
         fscanf (fptr, "%*d %*d");
         for (j = 0; j < nsnps; ++j)
         {
            fscanf (fptr, "%d", &(phasdatindiv[i][j]));
            if (-1 == phasdatindiv[i][j])
            {
               tmp1 = haplodat[2 * i][j];
               tmp2 = haplodat[2 * i + 1][j];
               haplodat[2 * i][j] = tmp2;
               haplodat[2 * i + 1][j] = tmp1;
               phasdatindiv[i][j] = 1;
            }
         }
      }
      for (i = 0; i < nset; ++i)
      {
         for (j = 0; j < nsnps; ++j)
         {
            /* it's only phased for the HMM model if BOTH are phased */
            phasdat[i][j] = phasdatindiv[2 * i][j] * phasdatindiv[2 * i + 1][j];
         }
      }
      for (i = 0; i < nhap / 2; ++i)
      {
         printf ("\n indiv haps %d %d:", name[2 * i], name[2 * i + 1]);
         for (j = 0; j < 10; ++j)
            printf (" %d", phasdatindiv[i][j]);
      }
      for (i = 0; i < nset; ++i)
      {
         printf ("\n pair (4-haps) %d %d %d %d:", name[4 * i], name[4 * i + 1],
                 name[4 * i + 2], name[4 * i + 3]);
         for (j = 0; j < 10; ++j)
            printf (" %d", phasdat[i][j]);
      }
      printf ("\n now have read the real data \n \n");
      fflush (stdout);
   }

/* set input SNP info data file  */
   if (!(fptr2 = fopen (mrks_fname, "r")))
      give_up (ERR_qit);

   for (j = 0; j < nsnps; ++j)
      fscanf (fptr2, "%lf", &(snppos[j]));
   for (j = 0; j < nsnps - 1; ++j)
      snpdist[j] = snppos[j + 1] - snppos[j];
   for (j = 0; j < nsnps; ++j)
      fscanf (fptr2, "%*d %lf %*f", allefreq + j);

/* set output file */
   if (!(fptr = open_fil (OXTR)))
      give_up (ERR_qit);

/* make the data vector of states from the haplo-data */
/* first haplodat has switched from 0,1 to 1,2 (with 0 missing  */
/* second we can now do several groups of 4 haplotypes: for 1 set kk=0 */

   hh = 1 - fkin;
   gg = fkin;

   set_marg_ibd (ns, ffibd);            /* uses fkin; overall kinship */
   printf ("\n equilib state probs at kinship %f\n", fkin);
   for (i = 0; i < ns; ++i)
      printf (" %5.3f", ffibd[i]);
   fflush (stdout);

/* reset data for test */
   totset = nset;

   for (k = 0; k < totset; ++k)         /* k th pair of genos */
   {
      printf ("\n data vector for %d markers in set %d to be set", nloc, k + 1);
      fflush (stdout);
      make_data (hapdat, haplodat, nhd, k, nloc, 1, 0);

      printf ("\n data vector for %d markers in set %d is set", nloc, k + 1);
      fflush (stdout);

/*  check transmat */
      set_transmat (ns, transmat, ffibd, 1.0);
      for (i = 0; i < ns; ++i)
      {
         printf ("\n");
         for (j = 0; j < ns; ++j)
            printf (" %7.5f", transmat[i][j]);
      }

      printf ("\n markers %d, phen %d , states %d", nloc, nph, ns);

      /* if using the matrix of phase data, set the relevant pointer */
      if (igdx == 2)
      {
         phasptr = phasdat[k];
      }
      hap_fwd_compute (hapdat, nloc, nph, ns, transmat, allefreq,
                       ffibd, snpdist, pprob, igdx, phasptr);
      hap_bck_compute (hapdat, nloc, nph, ns, qq_ibd, transmat,
                       allefreq, ffibd, snpdist, pprob, igdx, phasptr);

      printf ("\n \n markers = %d\n", nloc);
      fflush (stdout);
      printf (" names %d %d \n", name[2 * k], name[2 * k + 1]);
      fprintf (fptr, "markers = %d \n", nloc);
      fflush (fptr);

      if (igin)
         fprintf (fptr, " individuals names %d %d \n",
                  name[2 * k], name[2 * k + 1]);
      else
      {
         if (!igdx)
            fprintf (fptr, " hap-names ");
         else
            fprintf (fptr, " hap-names; paired as genotypes");
         for (j = 0; j < 4; ++j)
            fprintf (fptr, " %d ", name[4 * k + j]);
         fprintf (fptr, " \n");
      }

      /* if partially phased data, collapse 15 states down to 9 */
      if (igdx == 2)
      {
         qq_ibd_final = alloc_dbl2d (nsnps, 9);
         for (i = 0; i < nloc; ++i)
         {
            qq_ibd_final[i][0] = qq_ibd[i][0];
            qq_ibd_final[i][1] = qq_ibd[i][1];
            qq_ibd_final[i][2] = qq_ibd[i][2] + qq_ibd[i][3];
            qq_ibd_final[i][3] = qq_ibd[i][4];
            qq_ibd_final[i][4] = qq_ibd[i][5] + qq_ibd[i][6];
            qq_ibd_final[i][5] = qq_ibd[i][7];
            qq_ibd_final[i][6] = qq_ibd[i][8] + qq_ibd[i][9];
            qq_ibd_final[i][7] =
               qq_ibd[i][10] + qq_ibd[i][11] + qq_ibd[i][12] + qq_ibd[i][13];
            qq_ibd_final[i][8] = qq_ibd[i][14];
         }
         nstmp = 9;
      }
      else
      {
         qq_ibd_final = qq_ibd;
         nstmp = ns;
      }

      for (i = 0; i < nloc; ++i)
      {
         fprintf (fptr, " %d %f ", i, snppos[i]);
         for (j = 0; j < nstmp; ++j)
            fprintf (fptr, " %7.4f", qq_ibd_final[i][j]);
         fprintf (fptr, "\n");
      }
   }
   return EXIT_SUCCESS;
}
