/*  Program to do a basic simulation and analysis of IBD inference with LD */
/* Routines to make and set the artificially constructed ibd               */
/*  Contains print_chrom():  prints out fgl and junctions                  */
/*           read_chunks();  read-in of 4 chomos at fixed chunks           */
/*           make_chunks();  sets up the chomosome chunk structures        */
/*           read_chrom(); alternatively, structures are read in directly  */
/*           make_true_ibd(); constructs the true ibd patterns from 
                                         chomosome chunk structures        */
/*
*/

void print_chrom (Chromseglist * chromo)
/* print out the segments of fgl and junctions of a simulated chromosome */
{
   int   i;
   Chromseglist *cc;
   i = 0;

   cc = chromo;
   while (cc)
   {
      ++i;
      printf ("\n segment %d is fgl %d and ends at %f cM",
              i, cc->current->fgl, cc->current->endpos);
      cc = cc->next;
   }
   printf ("\n");

   return;
}

/*  new format read-in of 4 chromosomes at fixed chunks */
void read_chunks (FILE * fp, int nd,
                  int chtemp[4][200], double segend[200], int *s15, int *s9)
{
   int   i, k;

   if (nd > 4)
   {
      printf ("oops; too many chromosomes");
      fflush (stdout);
      exit (99);
   }

   for (i = 0; i < 200; ++i)
   {
      for (k = 0; k < 4; ++k)
         fscanf (fp, "%d", &(chtemp[k][i]));
      fscanf (fp, "%d", s15 + i);
      fscanf (fp, "%d", s9 + i);
      for (k = 0; k < 7; ++k)
         fscanf (fp, "%*d");
      segend[i] = (i + 1) * 0.5 + 0.000001;
   }

   return;

}

void make_chunks (Chromseglist * chdat, int *chtmp, double *sgend,
                  double totlen)
{
   int   i;

   i = 0;
   chdat->current = (Chromseg *) malloc (sizeof (Chromseg));
   alloc_chk (WHERE, chdat->current);
   chdat->current->fgl = chtmp[0];

   while (sgend[i] < totlen)
   {
      chdat->current->endpos = sgend[i];
      chdat->next = (Chromseglist *) malloc (sizeof (Chromseglist));
      alloc_chk (WHERE, chdat->next);
      chdat = chdat->next;
      chdat->current = (Chromseg *) malloc (sizeof (Chromseg));
      chdat->current->fgl = chtmp[i + 1];
      ++i;
   }
   chdat->current->endpos = totlen;
   chdat->next = NULL;

   return;
}

void read_chrom (FILE * fp, Chromseglist * chromo, double totlen)
/* read in a chromosome, instead of generating it */
{
   double segpos;
   int   fglc;

   segpos = 0.0;
   chromo->current = (Chromseg *) malloc (sizeof (Chromseg));
   alloc_chk (WHERE, chromo->current);
   fscanf (fp, "%d", &fglc);
   chromo->current->fgl = fglc;

   while (segpos < totlen)
   {
      fscanf (fp, "%lf", &segpos);
      chromo->current->endpos = segpos;
      chromo->next = (Chromseglist *) malloc (sizeof (Chromseglist));
      alloc_chk (WHERE, chromo->next);
      chromo = chromo->next;
      chromo->current = (Chromseg *) malloc (sizeof (Chromseg));
      fscanf (fp, "%d", &fglc);
      chromo->current->fgl = fglc;
   }
   chromo->current->endpos = totlen;
   chromo->next = NULL;

   return;

}

void make_true_ibd (Chromseglist * tr_ibd,
                    Chromseglist * chrom1, Chromseglist * chrom2, double totlen)
/* make list of true ibd segments from chrom1 and chrom2 in tr_ibd */
{
   Chromseglist *c1, *c2;
   double emin, e1, e2;                 /* endpos of current segments */
   int   first, f1, f2, ff, ce;         /* fgl of the chromos */
   c1 = chrom1;
   c2 = chrom2;

   tr_ibd->current = (Chromseg *) malloc (sizeof (Chromseg));
   alloc_chk (WHERE, tr_ibd->current);
   first = 1;
   f1 = c1->current->fgl;
   f2 = c2->current->fgl;
   if (f1 == f2)
      tr_ibd->current->fgl = 1;
   else
      tr_ibd->current->fgl = 0;

   while (c1 && c2)
   {
      e1 = c1->current->endpos;
      e2 = c2->current->endpos;
      if (first)
      {
         ff = tr_ibd->current->fgl;
         first = 0;
      }
      else
      {
         f1 = c1->current->fgl;
         f2 = c2->current->fgl;
         if (f1 == f2)
            ff = 1;
         else
            ff = 0;
         if (ff - (tr_ibd->current->fgl))
         {
            tr_ibd->current->endpos = emin;
            tr_ibd->next = (Chromseglist *) malloc (sizeof (Chromseglist));
            alloc_chk (WHERE, tr_ibd->next);
            tr_ibd = tr_ibd->next;
            tr_ibd->current = (Chromseg *) malloc (sizeof (Chromseg));
            alloc_chk (WHERE, tr_ibd->current);
            tr_ibd->current->fgl = ff;
         }
      }
      if (e1 < e2)
      {
         emin = e1;
         ce = 1;
      }
      else
      {
         emin = e2;
         ce = 2;
      }

      if (emin > totlen)
         break;

      if (ce == 1)
         c1 = c1->next;
      else
         c2 = c2->next;
   }

   tr_ibd->current->endpos = totlen;
   tr_ibd->next = NULL;

   return;
}
