/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

int cmpu_scor_iter (void)               /* compute scores every I iterations  */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 1)
         err_typ = ERR_SCI;
      ScorIt = j_par[0];
   }
   return err_typ;
}

int comp_eigen (void)                   /* compute eigenvalues statement      */
{
   if (chk_stat ())
   {
      chk_repeat ();
      ICh = 1;
   }
   return err_typ;
}

int check_consistency (void)            /* Check markers consistency.         */
{
   if (chk_stat ())
   {
      chk_repeat ();
      ChkMarksConsistent = TRUE;
      ChkConsOnly = FALSE;
   }
   return err_typ;
}

int check_consis_only (void)            /* Check markers consistency only.    */
{
   if (chk_stat ())
   {
      chk_repeat ();
      ChkMarksConsistent = TRUE;
      ChkConsOnly = TRUE;
   }
   return err_typ;
}

int check_eigen (void)                  /* check eigenvalues                  */
{
   if (chk_stat ())
   {
      chk_repeat ();
      ChkEig = Y;
   }
   return err_typ;
}

int check_ginv (void)                   /* check ginverse                     */
{
   if (chk_stat ())
   {
      chk_repeat ();
      ChkGnv = Y;
   }
   return err_typ;
}

int check_gmat (void)                   /* check gmatrix                      */
{
   if (chk_stat ())
   {
      chk_repeat ();
      ChkGmt = Y;
   }
   return err_typ;
}

int check_trace (void)                  /* check trace                        */
{
   if (chk_stat ())
   {
      chk_repeat ();
      ChkTrc = Y;
   }
   return err_typ;
}

int check_eig_cmp (void)                /* check eigenvalue computation       */
{
   if (chk_stat ())
   {
      chk_repeat ();
      ChkEigC = Y;
   }
   return err_typ;
}

int check_mc_iter (void)                /* check MC iterations statement      */
{
   if (chk_stat ())
   {
      chk_repeat ();
      ChkIt = j_par[0];
   }
   return err_typ;
}

int cmpu_kinshp (void)                  /* compute kinship coeff              */
{
   KINS *p_kinshp;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_kinshp (NCMP, 2))
      {
         if (!(p_kinshp = chk_kinshp_node (NCMP, KINSHP)))
            p_kinshp = alloc_kinshp_node (NCMP, KINSHP);
         else
            free_nams_list (p_kinshp);
         alloc_nams_list (p_kinshp);
         stor_kinshp (p_kinshp, NCMP, 2);
      }
   }
   return err_typ;
}

int cmpu_inbreed (void)                 /* compute inbreeding coeff           */
{
   KINS *p_inbr;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_kinshp (NCMP, 1))
      {
         if (!(p_inbr = chk_kinshp_node (NCMP, INBRD)))
            p_inbr = alloc_kinshp_node (NCMP, INBRD);
         else
            free_nams_list (p_inbr);
         alloc_nams_list (p_inbr);
         stor_kinshp (p_inbr, NCMP, 1);
      }
   }
   return err_typ;
}

int cmpu_2_loc (void)                   /* compute 2-locus inbreed coeff      */
{
   KINS *p_inbr2;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_kinshp (NCMP, 1))
      {
         if (!(p_inbr2 = chk_kinshp_node (NCMP, INBRD2)))
            p_inbr2 = alloc_kinshp_node (NCMP, INBRD2);
         else
            free_nams_list (p_inbr2);
         alloc_nams_list (p_inbr2);
         stor_kinshp (p_inbr2, NCMP, 1);
      }
   }
   return err_typ;
}

int cmpu_ckinshp (void)                 /* compute kinship coeff              */
{
   KINS *p_kinshp;

   if (chk_stat ())
      if (chk_kinshp (CMP, 2))
      {
         if (!(p_kinshp = chk_kinshp_node (CMP, KINSHP)))
            p_kinshp = alloc_kinshp_node (CMP, KINSHP);
         else
            free_nams_list (p_kinshp);
         alloc_nams_list (p_kinshp);
         stor_kinshp (p_kinshp, CMP, 2);
      }
   return err_typ;
}

int cmpu_cinbreed (void)                /* compute inbreeding coeff           */
{
   KINS *p_inbr;

   if (chk_stat ())
      if (chk_kinshp (CMP, 1))
      {
         if (!(p_inbr = chk_kinshp_node (CMP, INBRD)))
            p_inbr = alloc_kinshp_node (CMP, INBRD);
         else
            free_nams_list (p_inbr);
         alloc_nams_list (p_inbr);
         stor_kinshp (p_inbr, CMP, 1);
      }
   return err_typ;
}

int cmpu_c2_loc (void)                  /* compute 2-locus inbreed coeff      */
{
   KINS *p_inbr2;

   if (chk_stat ())
      if (chk_kinshp (CMP, 1))
      {
         if (!(p_inbr2 = chk_kinshp_node (CMP, INBRD2)))
            p_inbr2 = alloc_kinshp_node (CMP, INBRD2);
         else
            free_nams_list (p_inbr2);
         alloc_nams_list (p_inbr2);
         stor_kinshp (p_inbr2, CMP, 1);
      }
   return err_typ;
}

int cmpu_est_n (void)                   /* compute estimates I times          */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 1)
         err_typ = ERA_CET;
      RedoEst = j_par[0];
   }
   return err_typ;
}

int cmpu_ibdstat (void)                 /* compute ibd statistics             */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (IBDStats != UNDEF || LikRatStats != UNDEF)
         err_typ = ERA_RST;
      IBDStats = 1;
      LikRatStats = 0;
   }
   return err_typ;
}

int cmpu_likstat (void)                 /* compute likelihood-ratio stats     */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (IBDStats != UNDEF || LikRatStats != UNDEF)
         err_typ = ERA_RST;
      IBDStats = 0;
      LikRatStats = 1;
   }
   return err_typ;
}
