/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
#include "quant.h"

#define SS(r,c)  SigSq[(r) * (r - 1) / 2 + c - 1]
#define TS(r,c)  TauSq[(r) * (r - 1) / 2 + c - 1]

void poly_print_pars (int iter_no, int prog_flag)
{
   int   i, j, k;

   switch (prog_flag)
   {
       case 1:
       case 2:
       case 4:
       case 5:
          printf (" iteration #%d:\n\n", iter_no);
          break;

       default:
          break;
   }

   switch (prog_flag)
   {
       case 1:                         /* univar    */
       case 2:                         /* unibig    */
          printf ("    variance estimates");
          printf ("\n       additive       ");
          print_val (SigAdd);
          printf ("\n       residual       ");
          print_val (SigErr);
          break;

       case 4:                         /* bivar     */
          printf ("    variance estimates");
          printf (" (trait %d, covariance, trait %d)", TrNo[0], TrNo[1]);
          printf ("\n       additive       ");
          for (i = 0; i < 3; ++i)
             print_val (SigSq[i]);
          printf ("\n       residual       ");
          for (i = 0; i < 3; ++i)
             print_val (TauSq[i]);
          break;

       case 5:                         /* multivar  */
          if (NTraits == 1)
             printf ("    additive variance estimate (trait %d", TrNo[0]);
          else
             printf ("    additive variance estimates (traits %d", TrNo[0]);
          for (i = 1; i < NTraits; ++i)
             printf (", %d", TrNo[i]);
          printf (")\n    ");
          for (i = 1; i <= NTraits; ++i)
             print_val (SS (i, i));
          if (NTraits > 1)
          {
             if (IOCor)
                printf ("\n    correlations\n    ");
             else
                printf ("\n    covariances\n    ");
             for (i = 2; i <= NTraits; ++i)
             {
                for (j = 1; j < i; ++j)
                   if (IOCor)
                      if (SS (i, i) != 0.0 && SS (j, j) != 0.0)
                         print_val (SS (i, j) /
                                    (sqrt (SS (i, i)) * sqrt (SS (j, j))));
                      else
                         print_val (0.0);
                   else
                      print_val (SS (i, j));
                printf ("\n    ");
             }
          }
          printf ("\n");

       case 6:
          if (NTraits == 1)
             printf ("    residual variance estimate (trait %d", TrNo[0]);
          else
             printf ("    residual variance estimates (traits %d", TrNo[0]);
          for (i = 1; i < NTraits; ++i)
             printf (", %d", TrNo[i]);
          printf (")\n    ");
          for (i = 1; i <= NTraits; ++i)
             print_val (TS (i, i));
          if (NTraits > 1)
          {
             if (IOCor)
                printf ("\n    correlations\n    ");
             else
                printf ("\n    covariances\n    ");
             for (i = 2; i <= NTraits; ++i)
             {
                for (j = 1; j < i; ++j)
                   if (IOCor)
                      if (TS (i, i) != 0.0 && TS (j, j) != 0.0)
                         print_val (TS (i, j) / (sqrt (TS (i, i))
                                                 * sqrt (TS (j, j))));
                      else
                         print_val (0.0);
                   else
                      print_val (TS (i, j));
                printf ("\n    ");
             }
          }
          break;

       default:
          break;
   }

   for (k = 0; k < NTraits; ++k)
   {
      printf ("\n    trait %d", TrNo[k]);
      printf ("\n       overall mean   ");
      print_val (Mufti[k][0][1]);
      for (i = 1; i <= NFix; ++i)
         if (FfDex[k][i] != 0)
            for (j = 1; j <= IFix[i]; ++j)
            {
               if (j == 1)
                  printf ("\n       fixed effect %2d", FENo[i]);
               else if (j - 6 * (j / 6) == 1)
                  printf ("\n                      ");
               print_val (Mufti[k][i][j]);
            }
   }
   printf ("\n");
   fflush (stdout);

   return;
}
